'' =================================================================================================
''
''   File....... jm_etimer_v2.spin
''   Purpose.... Background timer
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009-2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 21 JUL 2009
''   Updated.... 22 JAN 2011
''
'' =================================================================================================

{{

   V2 :: Timer registers in parent object
         Changed days register to 0 to 6 (Sunday to Saturday)

}}

  
con

  #0, ET_MS, ET_SC, ET_MN, ET_HR, ET_DY,LT_MS, LT_SC
  

obj


var

  long  cog

  long  mstix                                                   ' ticks per ms
  long  timermode                                               ' 0 = hold, else run
  long  tregs                                                   ' address of timer regs
  long  hbpin                                                   ' heartbeat   


pub init(pntr, hb) | okay

'' Starts event timer driver
'' -- pntr is hub address of longs array for timing registers
'' -- hb is "heartbeat" pin (use -1 to disable)

  finalize                                                      ' stop if already running 

  mstix := clkfreq / 1_000                                      ' cnt ticks per millisecond
  longmove(@tregs, @pntr, 2)                                    ' copy params

  reset                                                         ' clear registers
  resume                                                        ' enable timer
   
  okay := cog := cognew(@etimer, @mstix) + 1                    ' start timer cog

  return okay


pub finalize

'' Stops event timer; frees a cog 

  if cog
    cogstop(cog~ - 1)
    reset 


pub read(pntr)

'' Copies timer registers to array at pntr

  longmove(pntr, tregs, 5)  


pub write(pntr)

'' Copies values in array at pntr to timer registers

  longmove(tregs, pntr, 5)


pub set(_ms, _sc, _mn, _hr, _dy)

'' Set individual clock registers

  longmove(tregs, @_ms, 5)
  
  
pub readreg(reg) | value

'' Reads selected register from the timer

  if (reg => ET_MS) and (reg =< ET_DY)
    value := long[tregs][reg]
  else
    value := -1

  return value

    
pub writereg(reg, value) | okay

'' Returns selected register from the timer

  if (reg => ET_MS) and (reg =< ET_DY) 
    long[tregs][reg] := value
    okay := true
  else
    okay := false

  return okay    


pub hold

'' Stops timer; does not clear

  timermode := false


pub resume

'' Restarts timer; does not clear 

  timermode := true


pub reset

'' Clears timer value; does not stop timer

  longfill(tregs, 0, 5)


dat

                        org     0

etimer                  mov     tmp1, par                       ' start of structure
                        rdlong  ms001, tmp1                     ' read ticks/ms
                        add     tmp1, #4
                        mov     mdpntr, tmp1                    ' save ponter to mode
                        add     tmp1, #4
                        rdlong  mspntr, tmp1                    ' read address of ms register
                        add     tmp1, #4
                        rdlong  tmp2, tmp1                      ' read hb pin
                        cmps    tmp2, #0                wc, wz  ' compare to 0

        if_b            mov     hbmask, #0                      ' no mask if < 0                     '
        if_ae           mov     hbmask, #1                      ' else make pin mask
        if_ae           shl     hbmask, tmp2                               
                        tjz     hbmask, #checkmode              ' hb enabled?
                        andn    outa, hbmask                    ' yes, clear pin
                        or      dira, hbmask                    ' make output

sync                    mov     ms1timer, ms001                 ' initialize timer
                        add     ms1timer, cnt                   ' sync
                                                                 
checkmode               rdlong  tmp1, mdpntr            wz      ' get timer mode
        if_z            jmp     #sync                           ' if 0, disable
                                                                 
                        waitcnt ms1timer, ms001                 ' else wait on 1ms timer
                        mov     regspntr, mspntr                ' start with ms register
                                                                 
getms                   rdlong  tmp1, regspntr                  ' read current ms
                                                                 
heartbeat               tjz     hbmask, #updatems               ' skip if no hb  
                        cmp     tmp1, #0                wz      ' if 0
        if_e            xor     outa, hbmask                    '   toggle
        if_e            jmp     #updatems                        
                        cmp     tmp1, #500              wz      ' or 500
        if_e            xor     outa, hbmask                    '   toggle
                                                                 
updatems                add     tmp1, #1                        ' increment
                        cmp     tmp1, SEC_01            wz      ' at limit?
        if_e            mov     tmp1, #0                        ' if yes, clear
                        wrlong  tmp1, regspntr                  ' write back new value
        if_ne           jmp     #checkmode                      ' to top if no roll-over
                                                                 
updatesecs              add     regspntr, #4                    ' point to seconds
                        rdlong  tmp1, regspntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wz
        if_e            mov     tmp1, #0
                        wrlong  tmp1, regspntr
        if_ne           jmp     #checkmode                            

updatemins              add     regspntr, #4                    ' point to mins
                        rdlong  tmp1, regspntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wz
        if_e            mov     tmp1, #0
                        wrlong  tmp1, regspntr
        if_ne           jmp     #checkmode

updatehrs               add     regspntr, #4                    ' point to hours       
                        rdlong  tmp1, regspntr
                        add     tmp1, #1
                        cmp     tmp1, #24               wz
        if_e            mov     tmp1, #0
                        wrlong  tmp1, regspntr
        if_ne           jmp     #checkmode

updateday               add     regspntr, #4                    ' point to mins
                        rdlong  tmp1, regspntr
                        add     tmp1, #1
                        cmp     tmp1, #7                wz
        if_e            mov     tmp1, #0
                        wrlong  tmp1, regspntr
        if_ne           jmp     #checkmode
                
' --------------------------------------------------------------------------------------------------

SEC_01                  long    1_000

ms001                   res     1                               ' cnt ticks per millisecond
mdpntr                  res     1                               ' hub address of timer mode
mspntr                  res     1                               ' hub address of timer array
hbmask                  res     1                               ' heartbeat pin mask

regspntr                res     1                               ' working pointer

ms1timer                res     1                               ' for 1ms timing

tmp1                    res     1
tmp2                    res     1

                        fit     
                        

dat

{{

  Terms of Use: MIT License 

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   